-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

--global values
local currentWeight
local currentWeightMultiplier
local currentStiffnessMultiplier
local weightEditStep
local stiffnessEditStep
local sizeEditStep
local nodesAmount
local minWeightLimitReached

local teleportX
local teleportY
local teleportZ

local function init(jbeamData)

  --get jbeamData
  weightEditStep = jbeamData.weightEditStep or 10
  stiffnessEditStep = weightEditStep > 100 and 100 or weightEditStep --can't gain too much stiffness or it will explode
  sizeEditStep = jbeamData.sizeEditStep or 0.001
  
  teleportX = jbeamData.teleportX or 0
  teleportY = jbeamData.teleportY or 0
  teleportZ = jbeamData.teleportZ or 0
  
  --reset multipliers
  currentWeight = 0
  currentWeightMultiplier = 1
  currentStiffnessMultiplier = 1
  nodesAmount = 0
  minWeightLimitReached = false
  
  --get weight for each node and full car weight
  for _, node in pairs (v.data.nodes) do
	node.originalWeight = node.nodeWeight --save in new parameter
	currentWeight = currentWeight + node.originalWeight
	nodesAmount = nodesAmount + 1
  end
  
  --set stiffness for all beams
  for _, beam in pairs (v.data.beams) do
	beam.originalSpring = beam.beamSpring
	beam.originalDamp = beam.beamDamp
  end
  
  --same for torsionbars
  for _, torsionbar in pairs (v.data.torsionbars) do
	torsionbar.originalSpring = torsionbar.spring
	torsionbar.originalDamp = torsionbar.damp
  end
  
end

local function updateMass()

  minWeightLimitReached = false

  --update node weight
  currentWeight = 0
  for _, node in pairs (v.data.nodes) do
  
    if obj:getNodeMass(node.cid) <= weightEditStep / nodesAmount then --min safe weight reached
	  minWeightLimitReached = true
	  return
	end
  
	obj:setNodeMass(node.cid, node.originalWeight * currentWeightMultiplier)
	currentWeight = currentWeight + node.originalWeight * currentWeightMultiplier
  end
  
  --update beam spring and damp for correct stiffness
  for _, beam in pairs (v.data.beams) do
	obj:setBeamSpringDamp(beam.cid, beam.originalSpring * currentStiffnessMultiplier, beam.originalDamp * currentStiffnessMultiplier, -1, -1)	
  end
  
  --same for torsion bars
  for _, torsionbar in pairs (v.data.torsionbars) do
	obj:setTorsionbarSpringDamp(torsionbar.cid, torsionbar.originalSpring * currentStiffnessMultiplier, torsionbar.originalDamp * currentStiffnessMultiplier)	
  end
  
  --send gui message
  guihooks.message("Current weight (without fuel): " .. currentWeight .. "kg", 5, "agenty_weird_vehicle_controls")
  
end

local function massUp(VALUE)
  if VALUE < 0.9 then return end -- axis/button is not being pressed down far enough, ignore this event

  --increase mass of all nodes
  currentWeightMultiplier = currentWeightMultiplier + weightEditStep / nodesAmount
  currentStiffnessMultiplier = currentStiffnessMultiplier + stiffnessEditStep / nodesAmount
  updateMass()
  
end

local function massDn(VALUE)
  if VALUE < 0.9 then return end -- axis/button is not being pressed down far enough, ignore this event

  --decrease mass of all nodes
  currentWeightMultiplier = currentWeightMultiplier - weightEditStep / nodesAmount
  currentStiffnessMultiplier = currentStiffnessMultiplier - stiffnessEditStep / nodesAmount
  updateMass()
  
end

local function sizeUp(VALUE)
  if VALUE < 0.9 then return end -- axis/button is not being pressed down far enough, ignore this event
  
  for _, beam in pairs (v.data.beams) do
	obj:setBeamLength(beam.cid, obj:getBeamLength(beam.cid) * (sizeEditStep + 1))
  end 
  
  --increase tire pressure
  for _, pressureGroup in pairs (v.data.pressureGroups) do
	obj:setGroupPressure(pressureGroup, obj:getGroupPressure(pressureGroup) * (sizeEditStep + 1))
  end
  for _, wheel in pairs (v.data.wheels) do
	if wheel.pressuredBeams ~= nil then
      for _, beam in pairs(wheel.pressuredBeams) do
        obj:setBeamPressureRel(beam.cid, beam.pressurePSI * psiToPascal * (sizeEditStep + 1), math.huge, -1, -1)
      end
    end
  end
  
end

local function sizeDn(VALUE)
  if VALUE < 0.9 then return end -- axis/button is not being pressed down far enough, ignore this event
  
  for _, beam in pairs (v.data.beams) do
    if obj:getBeamLength(beam.cid) <= sizeEditStep then
	  guihooks.message("Reached minimum safe size, cannot decrease further!", 5, "agenty_weird_vehicle_controls")
	  return
    end
	obj:setBeamLength(beam.cid, obj:getBeamLength(beam.cid) / (sizeEditStep + 1))
  end  
  
  --decrease tire pressure
  for _, pressureGroup in pairs (v.data.pressureGroups) do
	obj:setGroupPressure(pressureGroup, obj:getGroupPressure(pressureGroup) / (sizeEditStep + 1))
  end
  for _, wheel in pairs (v.data.wheels) do
	if wheel.pressuredBeams ~= nil then
      for _, beam in pairs(wheel.pressuredBeams) do
        obj:setBeamPressureRel(beam.cid, beam.pressurePSI * psiToPascal / (sizeEditStep + 1), math.huge, -1, -1)
      end
    end
  end
  
end

local function teleport()
  for cid, node in pairs (v.data.nodes) do
    local d = obj:getNodePosition(node.cid):toTable()
	d[1] = d[1] + teleportX
	d[2] = d[2] + teleportY
	d[3] = d[3] + teleportZ
	obj:setNodePosition(cid, vec3(d))
  end
end

-- public interface

M.init          = init
M.updateMass    = updateMass
M.massUp        = massUp
M.massDn        = massDn
M.sizeUp        = sizeUp
M.sizeDn        = sizeDn
M.teleport      = teleport

return M
